@@ -566,7 +566,7 @@ class Subprocess:
     def set_uid(self):
         if self.config.uid is None:
             return
-        msg = self.config.options.dropPrivileges(self.config.uid)
+        msg = self.config.options.drop_privileges(self.config.uid)
         return msg
 
     def __cmp__(self, other):
@@ -755,6 +755,9 @@ class ProcessGroupBase:
             dispatchers.update(process.dispatchers)
         return dispatchers
 
+    def before_remove(self):
+        pass
+
 class ProcessGroup(ProcessGroupBase):
     def transition(self):
         for proc in self.processes.values():
@@ -778,12 +781,10 @@ class EventListenerPool(ProcessGroupBase):
     def __init__(self, config):
         ProcessGroupBase.__init__(self, config)
         self.event_buffer = []
-        for event_type in self.config.pool_events:
-            events.subscribe(event_type, self._acceptEvent)
-        events.subscribe(events.EventRejectedEvent, self.handle_rejected)
         self.serial = -1
         self.last_dispatch = 0
         self.dispatch_throttle = 0 # in seconds: .00195 is an interesting one
+        self._subscribe()
 
     def handle_rejected(self, event):
         process = event.process
@@ -809,6 +810,9 @@ class EventListenerPool(ProcessGroupBase):
                     return
             self.dispatch()
 
+    def before_remove(self):
+        self._unsubscribe()
+
     def dispatch(self):
         while self.event_buffer:
             # dispatch the oldest event
@@ -832,8 +836,9 @@ class EventListenerPool(ProcessGroupBase):
             event.pool_serials[self.config.name] = new_serial(self)
         else:
             self.config.options.logger.debug(
-                'rebuffering event %s for pool %s (bufsize %s)' % (
-                (event.serial, self.config.name, len(self.event_buffer))))
+                'rebuffering event %s for pool %s (buf size=%d, max=%d)' % (
+                (event.serial, self.config.name, len(self.event_buffer),
+                self.config.buffer_size)))
 
         if len(self.event_buffer) >= self.config.buffer_size:
             if self.event_buffer:
@@ -892,7 +897,18 @@ class EventListenerPool(ProcessGroupBase):
                 'pool:%(pool_name)s poolserial:%(pool_serial)s '
                 'eventname:%(event_name)s len:%(len)s\n%(payload)s' % D)
 
-class GlobalSerial:
+    def _subscribe(self):
+        for event_type in self.config.pool_events:
+            events.subscribe(event_type, self._acceptEvent)
+        events.subscribe(events.EventRejectedEvent, self.handle_rejected)
+
+    def _unsubscribe(self):
+        for event_type in self.config.pool_events:
+            events.unsubscribe(event_type, self._acceptEvent)
+        events.unsubscribe(events.EventRejectedEvent, self.handle_rejected)
+
+
+class GlobalSerial(object):
     def __init__(self):
         self.serial = -1
 
@@ -903,6 +919,3 @@ def new_serial(inst):
         inst.serial = -1
     inst.serial += 1
     return inst.serial
-
-
-
