@@ -1044,6 +1044,23 @@ class TestDefaultControllerPlugin(unittest.TestCase):
         plugin.do_version(None)
         self.assertEqual(plugin.ctl.stdout.getvalue(), '3000\n')
 
+    def test_version_arg(self):
+        plugin = self._makeOne()
+        result = plugin.do_version('bad')
+        self.assertEqual(result, None)
+        val = plugin.ctl.stdout.getvalue()
+        self.assertTrue(val.startswith('Error: version accepts no arguments'), val)
+
+    def test_version_upcheck_failed(self):
+        plugin = self._makeOne()
+        plugin.ctl.upcheck = lambda: False
+        called = []
+        def f(*arg, **kw):
+            called.append(True)
+        plugin.ctl.options._server.supervisor.getSupervisorVersion = f
+        plugin.do_version('')
+        self.assertEqual(called, [])
+
     def test_reload_help(self):
         plugin = self._makeOne()
         plugin.help_reload()
@@ -1065,6 +1082,13 @@ class TestDefaultControllerPlugin(unittest.TestCase):
         self.assertEqual(result, None)
         self.assertEqual(options._server.supervisor._restarted, True)
 
+    def test_reload_arg(self):
+        plugin = self._makeOne()
+        result = plugin.do_reload('bad')
+        self.assertEqual(result, None)
+        val = plugin.ctl.stdout.getvalue()
+        self.assertTrue(val.startswith('Error: reload accepts no arguments'), val)
+
     def test_shutdown_help(self):
         plugin = self._makeOne()
         plugin.help_shutdown()
@@ -1166,6 +1190,13 @@ class TestDefaultControllerPlugin(unittest.TestCase):
         self.assertEqual(result, None)
         self.assertEqual(calls[0], [['added'], ['changed'], ['removed']])
 
+    def test_reread_arg(self):
+        plugin = self._makeOne()
+        result = plugin.do_reread('bad')
+        self.assertEqual(result, None)
+        val = plugin.ctl.stdout.getvalue()
+        self.assertTrue(val.startswith('Error: reread accepts no arguments'), val)
+
     def test_reread_cant_reread(self):
         plugin = self._makeOne()
         from supervisor import xmlrpc
@@ -1234,6 +1265,13 @@ class TestDefaultControllerPlugin(unittest.TestCase):
         result = plugin.do_avail('')
         self.assertEqual(result, None)
 
+    def test_avail_arg(self):
+        plugin = self._makeOne()
+        result = plugin.do_avail('bad')
+        self.assertEqual(result, None)
+        val = plugin.ctl.stdout.getvalue()
+        self.assertTrue(val.startswith('Error: avail accepts no arguments'), val)
+
     def test_avail_shutdown_state(self):
         plugin = self._makeOne()
         supervisor = plugin.ctl.options._server.supervisor
@@ -1630,32 +1668,42 @@ class TestDefaultControllerPlugin(unittest.TestCase):
         result = plugin.do_fg('')
         self.assertEqual(result, None)
         lines = plugin.ctl.stdout.getvalue().split('\n')
-        self.assertEqual(lines[0], 'Error: no process name supplied')
+        self.assertEqual(lines[0], 'ERROR: no process name supplied')
 
     def test_fg_too_many_args(self):
         plugin = self._makeOne()
         result = plugin.do_fg('foo bar')
         self.assertEqual(result, None)
         line = plugin.ctl.stdout.getvalue()
-        self.assertEqual(line, 'Error: too many process names supplied\n')
+        self.assertEqual(line, 'ERROR: too many process names supplied\n')
 
     def test_fg_badprocname(self):
         plugin = self._makeOne()
         result = plugin.do_fg('BAD_NAME')
         self.assertEqual(result, None)
         line = plugin.ctl.stdout.getvalue()
-        self.assertEqual(line, 'Error: bad process name supplied\n')
+        self.assertEqual(line, 'ERROR: bad process name supplied\n')
 
     def test_fg_procnotrunning(self):
         plugin = self._makeOne()
         result = plugin.do_fg('bar')
         self.assertEqual(result, None)
         line = plugin.ctl.stdout.getvalue()
-        self.assertEqual(line, 'Error: process not running\n')
+        self.assertEqual(line, 'ERROR: process not running\n')
         result = plugin.do_fg('baz_01')
         lines = plugin.ctl.stdout.getvalue().split('\n')
         self.assertEqual(result, None)
-        self.assertEqual(lines[-2], 'Error: process not running')
+        self.assertEqual(lines[-2], 'ERROR: process not running')
+
+    def test_fg_upcheck_failed(self):
+        plugin = self._makeOne()
+        plugin.ctl.upcheck = lambda: False
+        called = []
+        def f(*arg, **kw):
+            called.append(True)
+        plugin.ctl.options._server.supervisor.getProcessInfo = f
+        plugin.do_fg('foo')
+        self.assertEqual(called, [])
 
     def test_exit_help(self):
         plugin = self._makeOne()
